flattenMCMC = function( samples ) {
  return(data.frame(do.call('rbind',samples)))
}

plotBayesianParameter = function( postSamples, parameterName, type='values', parameterLabels = NULL, credMass = 0.95, compVal = NULL, ROPE = NULL ) {
  oldrows = par('mfrow')
  match.arg(type,choices = c('values','differences'))
  
  postnms = names(postSamples)
  pnlen = nchar(parameterName)
  goodnms = postnms[substring(postnms,1,pnlen) == parameterName]
  nVals = length(goodnms)
  
  if(nVals == 0) {writeLines(paste(parameterName,'is not an acceptable parameter name'));return(invisible(NULL))}
  if(is.null(parameterLabels)) {parameterLabels = goodnms}
  if(length(parameterLabels) != nVals) {
    warning('Length of parameterLables does not match levels of parameters; setting to default names')
    parameterLabels = goodnms
  }
  
  pDF = postSamples[goodnms]
  
  
  if(nVals == 1) {plotPost(pDF[[parameterName]],credMass=credMass,compVal=compVal,ROPE=ROPE,main=parameterLabels,xlab=''); return(invisible(NULL))}
  
  if(type == 'values') {
    if(nVals > 4) {par(mfrow = c(floor(sqrt(nVals)),ceiling(nvals/floor(sqrt(nVals)))))}
    else {par(mfrow = c(1,nVals))}
    for(i in 1:nVals) {plotPost(pDF[[goodnms[i]]],credMass=credMass,compVal=compVal,ROPE=ROPE,main=parameterLabels[i],xlab='')}
  } else {
    par(mfrow = c(nVals,nVals))
    for(i in 1:nVals) {
      for(j in 1:nVals) {
        if(i == j) {
          plot.new()
          text(.5,.5,parameterLabels[i],cex = 3)
          box('figure')
        } else {
          diff = pDF[[goodnms[i]]]  - pDF[[goodnms[j]]]
          if(nVals > 20) {lb = ''} else {lb = paste(parameterLabels[i],'-',parameterLabels[j])}
          plotPost(diff,credMass=credMass,compVal=compVal,ROPE=ROPE,main=lb,xlab='')
          box('figure')
        }
      }
    }
  }
  par(mfrow = oldrows)
  return(invisible(NULL))
}
