# Plotting functions
diagnostic.plot = function(lm.obj, type = "all", predictor = NULL) {
  priask = par('ask')
  if(type == 'qq') {plot(lm.obj,which=2)}
  else if(type == 'residuals') {hist(resid(lm.obj),xlab = 'Residual Error',main = 'Distribution of Residuals')}
  else if(type == 'influence') {plot(lm.obj,which=4)}
  else if(type == 'fitted') {plot(lm.obj,which=1)}
  else if(type == 'time') {
    # Basic plot
    resids = resid(lm.obj)
    plot(resids,main = 'Residuals vs. Order',xlab = 'Time Position',ylab = 'Residuals')
    # Add trendline & 0 line
    panel.smooth(1:length(resids),resids)
    abline(h = 0, lty = 3, col = "gray")
  }
  else if(type == 'predictor' | type == 'abs predictor') {
    # Parse the different predictors
    prednms = dimnames(lm.obj$model)[[2]][-1]
    # If null - run through all predictors
    if(is.null(predictor)) {return(diagnostic.plot(lm.obj,type,prednms))}
    # Otherwise check that called predictors are in the model
    if(!(all(predictor %in% prednms))) {writeLines('One or more predictors not in the lm model');return(invisible(NULL))}
    # If more than one predictor, run through them all
    if(length(predictor) > 1) {
      diagnostic.plot(lm.obj,type,predictor[1])
      par(ask=T)
      for(pnm in predictor[-1]){diagnostic.plot(lm.obj,type,pnm)}
      par(ask=priask)
      return(invisible(NULL))
    }
    # Finally draw each individual graph
    rs = resid(lm.obj)
    if(type == 'abs predictor') {rs = abs(rs)} # vs absolute value if called
    prval = lm.obj$model[,predictor]
    plot(prval,rs,main = paste('Residuals vs. ',predictor),xlab = predictor,ylab = 'Residuals')
    panel.smooth(prval,rs)
    abline(h=0,lty=3,col='gray')
    par(ask = priask)
    return(invisible(NULL))
  }
  else if(type == 'all') {
    diagnostic.plot(lm.obj,'residuals')
    par(ask = T)
    diagnostic.plot(lm.obj,'qq')
    diagnostic.plot(lm.obj,'fitted')
    diagnostic.plot(lm.obj,'predictor')
    diagnostic.plot(lm.obj,'time')
    diagnostic.plot(lm.obj,'influence')
  }
  par(ask = priask)
}

plot.logistic = function(log.obj, xlab = NULL, ylab = NULL, zlab = NULL, ...) {
  
  if(!('glm' %in% class(log.obj))) {print('Requires glm object');return(invisible(NULL))}
  if(log.obj$family$family != 'binomial' | log.obj$family$link != 'logit') {print('Requires binomial logit family');return(invisible(NULL))}
  
  mod = log.obj$model
  if (dim(mod)[2] > 3) {print('Requires single or two-variable logistic regression');return(invisible(NULL))}
  
  if(class(mod[[1]]) == 'factor') {
    depdat = as.integer(mod[[1]]) - 1
    depnms = levels(mod[[1]])
  } else {
    depdat = mod[[1]]
    depnms = NULL
  }
  
  # For one dependent variable
  if (dim(mod)[2] == 2) {
    xs = mod[[2]]
    mnx = min(xs) - .1*(max(xs) - min(xs))
    mxx = max(xs) + .1*(max(xs) - min(xs))
    
    if(is.null(xlab)) {xlab = dimnames(mod)[[2]][2]}
    if(is.null(ylab)) {ylab = dimnames(mod)[[2]][1]}
    
    prdat = data.frame(seq(from=mnx,to=mxx,length.out=100))
    names(prdat) = dimnames(mod)[[2]][2]
    prdat$pr = predict(log.obj,newdata=prdat,type='response')
    
    if (!is.null(depnms)) {
      plot(xs,depdat,xlab=xlab,ylab = ylab,yaxt = 'n', ...)
      axis(2,at=c(0,1),labels=depnms)
    } else {
      plot(xs,depdat,xlab=xlab,ylab=ylab, ...)
    }
    lines(prdat[,dimnames(mod)[[2]][2]],prdat$pr,lty=2)
  }
  
  # For two dependent variables
  if (dim(mod)[2] == 3) {
    is.fact = function(x) {return(class(x) == 'factor' | class(x) == 'character')}
    
    xs = mod[[2]]
    ys = mod[[3]]
    
    xnm = dimnames(mod)[[2]][2]
    ynm = dimnames(mod)[[2]][3]
    
    if(is.null(xlab)) {xlab = xnm}
    if(is.null(ylab)) {ylab = ynm}
    if(is.null(zlab)) {zlab = dimnames(mod)[[2]][1]}
    
    # Plot like single dependent variable but for different category levels
    if(xor(is.fact(xs),is.fact(ys))) {
      if(is.fact(xs)) {cat.var = xs; cont.var = ys;cat.nm = xnm; cont.nm = ynm;cat.lab = xlab;cont.lab = ylab}
      else {cat.var = ys; cont.var = xs; cat.nm = ynm; cont.nm = xnm;cat.lab = ylab;cont.lab=xlab}
      
      nfacts = length(levels(cat.var))
      mnx = min(cont.var) - .1*(max(cont.var) - min(cont.var))
      mxx = max(cont.var) + .1*(max(cont.var) - min(cont.var))
      if (!is.null(depnms)) {
        plot(cont.var,depdat,xlab=cont.nm,ylab = zlab,yaxt = 'n',pch=as.integer(cat.var), ...)
        axis(2,at=c(0,1),labels=depnms)
      } else {
        plot(cont.var,depdat,xlab=cont.nm,ylab=zlab,pch=as.integer(cat.var), ...)
      }
      for(i in 1:nfacts) {
        prdat = data.frame(seq(from=mnx,to=mxx,length.out=100))
        prdat = cbind(prdat,rep(levels(cat.var)[i],100))
        names(prdat) = c(cont.nm,cat.nm)
        prdat$pr = predict(log.obj,newdata=prdat,type='response')
        lines(prdat[,cont.nm],prdat$pr,lty=i)
      }
      legend('right',levels(cat.var),lty=1:nfacts,pch=1:nfacts,title=cat.lab)
    } 
    else if(is.fact(xs) & is.fact(ys)) {writeLines('Use barplots to print logistic ANOVAs');return(invisible(NULL))}
    # Two continuous variable plotting
    else {
      # Check to make sure 'rgl' package is there
      if(!('rgl' %in% rownames(installed.packages()))) {
        writeLines("Package 'rgl' required for two continuous independent variables")
        writeLines("Please use install.packages('rgl') and try again")
        return(invisible(NULL))
      } else {
        if(!(require('rgl'))) {
          writeLines("Something went wrong! Package 'rgl' should be here but could not be loaded")
          return(invisible(NULL))
        }
      }
    
      #mnx = min(xs) - .1*(max(xs) - min(xs))
      #mxx = max(xs) + .1*(max(xs) - min(xs))
      #mny = min(ys) - .1*(max(ys) - min(ys))
      #mxy = max(ys) + .1*(max(ys) - min(ys))
      mnx = min(xs); mxx = max(xs); mny = min(ys); mxy = max(ys)
    
      xps = seq(from=mnx,to=mxx,length.out=30)
      yps = seq(from=mny,to=mxy,length.out=30)
      prdat = data.frame(rep(xps,each=30),rep(yps,times=30))
      names(prdat) = c(xnm,ynm)
      prdat$pr = predict(log.obj,newdata=prdat,type='response')
    
      plot3d(mod[[2]],mod[[3]],depdat,xlab = xlab, ylab = ylab, zlab = zlab,axes = F,box=T)
      axis3d('x')
      axis3d('y')
      if(!is.null(depnms)) {axis3d('z',labels=depnms,nticks=1)} else {axis3d('z')}
      box3d()
      surface3d(xps,yps,xtabs(prdat$pr~prdat[[1]]+prdat[[2]]),front = 'lines',back = 'lines')
    }
  }
  
}
