\name{permute.test}
\alias{permute.test}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Permutation (Randomization) Test
}
\description{
Automates the shuffling and sampling of permutation tests (though technically uses randomization tests).
}
\usage{
permute.test(data, test.stat, shuffler, n = 1000, alternative = c("two.sided","less","greater"), try.parallel = c(TRUE,FALSE), ncores = getOption('mcCores',2))
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data}{a vector, list, or data frame of the observed data.}
  \item{test.stat}{a function that takes only the '\code{data}' as input and outputs a single value representing the test statistic of interest.}
  \item{shuffler}{a function that takes only the '\code{data}' as input and outputs the data in equivalent form, but shuffled according to the logic of the permutation test.}
  \item{n}{the number of samples to take. Defaults to 1000.}
  \item{alternative}{a character string specifying the alternative hypothesis, must be one of "\code{two.sided}" (default), "\code{greater}" or "\code{less}".}
  \item{try.parallel}{option to parallelize the bootstrapping. Defaults to \code{TRUE} for Unix platforms, but does not exist on Windows platforms and is therefore turned off.}
  \item{ncores}{number of cores to use for parallelization. Defaults to what is set in environment variable MC_CORES, or 2 cores if that is not set.}
}

\value{
Returns a "\code{permtest}" S3 object. The information in this object can be retrieved or displayed using the "\code{print}", "\code{plot}", "\code{get_p}", or "\code{get_stat}" commands.
}
\author{
Kevin A Smith}

\seealso{
\code{\link{bootstrap.CI}}}
\examples{
# Doing a permutation version of a two-sample t-test on the memory.exp data

data(examples,package = 'PSYC201')

# Define the test statistic - the difference in the means should be 0
meandiff = function(dat) {
  x.bar = mean(dat$Score[dat$Group=='Experimental'])
  y.bar = mean(dat$Score[dat$Group=='Control'])
  return (x.bar - y.bar)
}

# Define the shuffling function
shuf.dat = function(dat) {
  dat$Score = sample(dat$Score,length(dat$Score),replace=FALSE)
  return(dat)
}

# Run the permutation test
ptest = permute.test(memory.exp,meandiff,shuf.dat)
print(ptest)

# And plot it
plot(ptest)
}